# SPDX-FileCopyrightText: 2025-present Wytamma Wirth <wytamma.wirth@me.com>
#
# SPDX-License-Identifier: MIT
import sys

if __name__ == "__main__":
    from beast_skygrid.cli import skygrid

    sys.exit(skygrid())



from pathlib import Path
import typer

def main(alignment: Path, output: Path):
    samples = []
    with open(alignment) as f:
        for line in f:
            if line.startswith(">"):
                samples.append(line.strip().lstrip(">"))
    with open(output, "w") as f:
        f.write("strain\tdate\n")
        for seq_name in samples:
            date = seq_name.split("|")[-1]
            if "/" in date:
                # ignore uncertainty in the date
                date = date.split("/")[0]
            f.write(f"{seq_name}\t{date}\n")

if __name__ == "__main__":
    typer.run(main)


from datetime import datetime, timedelta
from dataclasses import dataclass


def decimal_year_to_date(decimal_year):
    """
    Converts a decimal year to a date in the format '%Y-%m-%d'.

    Args:
      decimal_year (float): The decimal year to convert.

    Returns:
      str: The date in the format '%Y-%m-%d'.

    Examples:
      >>> decimal_year_to_date(2020.5)
      '2020-07-02'
    """
    year = int(decimal_year)
    remainder = decimal_year - year
    start_of_year = datetime(year, 1, 1)
    end_of_year = datetime(year + 1, 1, 1)
    days_in_year = (end_of_year - start_of_year).days
    days = remainder * days_in_year
    date = start_of_year + timedelta(days=days)
    return date.strftime('%Y-%m-%d')

def date_to_decimal_year(date_str):
    """
    Converts a date in the format '%Y-%m-%d' to a decimal year.

    Args:
      date_str (str): The date in the format '%Y-%m-%d'.

    Returns:
      float: The decimal year.

    Examples:
      >>> date_to_decimal_year('2020-07-02')
      2020.5
    """
    date = datetime.strptime(date_str, '%Y-%m-%d')
    year = date.year
    start_of_year = datetime(year, 1, 1)
    end_of_year = datetime(year + 1, 1, 1)
    days_in_year = (end_of_year - start_of_year).days
    days_passed = (date - start_of_year).days
    decimal_year = year + days_passed / days_in_year
    return decimal_year


@dataclass
class Taxon:
    """
    Dataclass representing a taxon.

    Attributes:
      id (str): The id of the taxon.
      sequence (str): The sequence of the taxon.
      date (float): The date of the taxon.
      uncertainty (float): The uncertainty of the taxon's date.
    """
    id: str
    sequence: str
    date: float
    uncertainty: float = 0.0


def taxa_from_fasta(fasta_path, date_delimiter="|", date_index=-1):
    """
    Parses a fasta file into a list of Taxon objects.

    Args:
      fasta_path (Path): The path to the fasta file.
      date_delimiter (str): The delimiter for the date in the fasta header.
      date_index (int): The index of the date in the fasta header.

    Returns:
      List[Taxon]: A list of Taxon objects representing the taxa in the fasta file.

    Raises:
      ValueError: If the fasta file is invalid.
    """
    # Read the fasta file
    with open(fasta_path) as fasta_file:
        fasta_lines = fasta_file.readlines()

    # check if valid fasta
    if not fasta_lines[0].startswith(">"):
        raise ValueError("Invalid fasta file.")
    # Parse the fasta file into Taxon objects. Support multi-line sequences.
    taxa = []
    for line in fasta_lines:
        if line.startswith(">"):
            header = line[1:].strip()
            # 1992-XX-XX
            date_with_uncertainty = header.split(date_delimiter)[date_index].lower().strip()
            uncertainty = None
            if date_with_uncertainty.endswith("-xx-xx"):
                uncertainty = 1 # 1 year
            elif date_with_uncertainty.endswith("-xx"):
                uncertainty = 0.0833 # 1 month
            date = date_with_uncertainty.replace("xx", "01")
            try:
                date = date_to_decimal_year(date)
            except ValueError:
                raise ValueError(f"Invalid date in header: {header}")
            taxa.append(Taxon(id=header, sequence="", date=float(date), uncertainty=uncertainty))
        else:
            taxa[-1].sequence += line.strip()

    return taxa


#!/usr/bin/env python3
from pathlib import Path
from typing_extensions import Annotated

from jinja2 import StrictUndefined, Template
from beast_skygrid.workflow.utils import taxa_from_fasta
import typer

def populate_template(
        template_path: Annotated[Path, typer.Argument(help="Path to the input Beast template file.", exists=True, file_okay=True, dir_okay=False)],
        alignment_path: Annotated[Path, typer.Argument(help="Path to the input alignment file.", exists=True, file_okay=True, dir_okay=False)],
        dimensions: Annotated[int, typer.Option(help="Number of dimensions in the SkyGrid model.")],
        cutoff: Annotated[float, typer.Option(help="Estiamted tMRCA of the tree.")],
        output: Annotated[Path, typer.Option(help="Path to the output Beast XML file.", dir_okay=False, writable=True)] = None,
        clock: Annotated[str, typer.Option(help="Clock model to use in the analysis.")] = "strict",
        relaxed_gamma_shape: Annotated[float, typer.Option(help="Shape parameter of the gamma distribution for the relaxed clock model.")] = 0.3,
        relaxed_gamma_scale: Annotated[float, typer.Option(help="Scale parameter of the gamma distribution for the relaxed clock model.")] = 0.001,
        chain_length: Annotated[int, typer.Option(help="Length of the MCMC chain.")] = 100000000,
        samples: Annotated[int, typer.Option(help="Number of samples to draw from the MCMC chain.")] = 10000,
        date_delimiter: Annotated[str, typer.Option(help="Delimiter for the date in the fasta header.")] = "|",
        date_index: Annotated[int, typer.Option(help="Index of the date in the fasta header.")] = -1,
        constant_sites: Annotated[str, typer.Option(help="Constant sites in format 'As Ts Gs Cs'.")] = None,
        trace: Annotated[bool, typer.Option(help="Whether to enable the trace log.")] = True,
        trees: Annotated[bool, typer.Option(help="Whether to enable the trees log.")] = True,
        sample_from_prior: Annotated[bool, typer.Option(help="Whether to sample from the prior.")] = False,
        fixed_clock_rate: Annotated[float, typer.Option(help="Fixed clock rate to use in the analysis.")] = None,
    ):
    """
    Populates a BEAST template with the given parameters.
    """
    # Load the template
    template = Template(template_path.read_text(), undefined=StrictUndefined)

    # Parse the alignment file into Taxon objects
    taxa = taxa_from_fasta(alignment_path, date_delimiter=date_delimiter, date_index=date_index)

    log_every = max(1, chain_length // samples)

    trace_log_every = log_every
    trace_log_name = "skygrid.log"

    tree_log_every = log_every
    tree_log_name = "skygrid.trees"

    # Render the template
    rendered_template = template.render(
        taxa=taxa,
        clockModel=clock,
        relaxedGammaShape=relaxed_gamma_shape,
        relaxedGammaScale=relaxed_gamma_scale,
        chainLength=chain_length,
        screenLogEvery=log_every,
        traceLogEvery=trace_log_every,
        traceLogName=trace_log_name,
        treeLogEvery=tree_log_every,
        treeLogName=tree_log_name,
        dimensions=dimensions,
        cutoff=cutoff,
        constantSites=constant_sites,
        sampleFromPrior=sample_from_prior,
        fixedClockRate=fixed_clock_rate,
    )
    if output:
        # Write the rendered template to a file
        output.write_text(rendered_template)
    else:
        # Print the rendered template to the console
        print(rendered_template)


if __name__ == "__main__":
    typer.run(populate_template)